// NRF24L01+ demo in C for the NXP LPC1114FN28
// Author: Frank Duignan.  
// Updates posted on http://ioprog.com and http://eleceng.dit.ie/frank
#include "stm32f030xx.h"
#include "nrf24l01.h"
#include "spi.h"
#define PAYLOAD_LENGTH 10
uint8_t SOURCE_ADDRESS[]={0x0a,1,1,1,4};
uint8_t DESTINATION_ADDRESS[]={8,0,5,8,6};
static int NRFStatus=0; // Update this each time the Slave Select line goes low
void NRFWriteCE(int Value)
{	
	// CE is on GPIOA bit 13
	if (Value)
		GPIOA_ODR |= BIT13;
	else
		GPIOA_ODR &= ~BIT13;
		
}
void NRFWriteCSN(int Value)
{
	// CSN is on PB1
	if (Value)
		GPIOB_ODR |= BIT1;
	else
		GPIOB_ODR &= ~BIT1;
		
}
void NRFinit()
{
	char Address[5];
	delay(100000); // wait 100ms
	initSPI(8,0);
	NRFWriteCE(0);					// Turn off transmitter/receiver for the moment.
	NRFWriteCSN(1);					// CSN Initially high
	delay(100000); 					// wait 100ms
	NRFWriteRegister(0,0);			// Clear out config
	NRFWriteRegister(1,3);			// Enable auto ack on P0 and P1
	NRFWriteRegister(4,0x5f);		// 1500 microseconds between retries, up to 15 retries
	NRFWriteRegister(2,	3);			// Enable data pipes 0 and 1
	NRFWriteRegister(3,3);			// Choose 5 byte addressing	
	NRFWriteRegister(0x05,100);		// select channel 100 (above WiFi)
	NRFWriteRegister(0x06,0x26); 	// select 250kbps, maximum power
	NRFWriteRegister(0x07,0x70);	// clear flags in status register		
	NRFWriteRegister(0x11,0);		// Auto ack pipe
	NRFWriteRegister(0x12,PAYLOAD_LENGTH);		// Set payload length	
	NRFSetRXAddress0(5,DESTINATION_ADDRESS);	// set address for auto ack	
	NRFSetTXAddress(5,DESTINATION_ADDRESS);		// set address for TX	
	NRFSetRXAddress1(5,SOURCE_ADDRESS); // set address for rx
	NRFEnableRXMode();				// power up NRF as receiver.
	delay(100000); 					// wait 100ms	
}

int NRFReadRegister(int RegNum)
{
	int ReturnValue=0;
	NRFWriteCSN(0);	
	//delay(10); // wait 10us
	if (RegNum < 0x20)
	{
		NRFStatus = transferSPI(RegNum); // return status in high byte						
		//delay(10);
		ReturnValue = transferSPI(0xff);	      // Send dummy byte to generate clocks
		//delay(10);
	}
	else
	{
		ReturnValue = -1;
	}	
	NRFWriteCSN(1);	
	//delay(10);
	return ReturnValue;
}
int NRFWriteRegister(int RegNum, uint8_t Value)
{
	int ReturnValue=0;	
	NRFWriteCSN(0);	
	//delay(10);
	if (RegNum < 0x20)
	{
		
		NRFStatus = transferSPI(0x20+RegNum); // return status in high byte				
		//delay(10);
		ReturnValue = transferSPI(Value);	 	       // Write byte to target
		//delay(10);
	}
	else
	{
		ReturnValue = -1;
	}
	NRFWriteCSN(1);	
	//delay(10);
	return ReturnValue;
}
int NRFGetStatus()
{
	return NRFStatus;
}
void NRFFlushTX(void)
{	
	NRFWriteCSN(0);	
	NRFStatus = transferSPI(0xe1); // Send Flush TX command
	NRFWriteCSN(1);	
}

void NRFFlushRX(void)
{	
	NRFWriteCSN(0);	
	NRFStatus = transferSPI(0xe2); // Send Flush RX command
	NRFWriteCSN(1);	
}

int NRFSetRXAddress0(int AddressLength, uint8_t * Address)
{
	int index;
	switch (AddressLength) {
		case 3 : {
			NRFWriteRegister(3,1); // 3 byte address length			
			break;
		}
		case 4 : {
			NRFWriteRegister(3,2); // 4 byte address length
			break;
		}
		case 5 : {
			NRFWriteRegister(3,3); // 5 byte address length
			break;
		}
		default: {
			return -1; // invalid address length
		}
	}
	NRFWriteCSN(0);	
	delay(10);
	NRFStatus = transferSPI(0x20+0x0a); // start write to RX_P0_Pipe address
	for (index = 0; index < AddressLength; index++)
	{
		NRFStatus = transferSPI(Address[index]);
		delay(10);
	}
	NRFWriteCSN(1);	
}

int NRFSetRXAddress1(int AddressLength, uint8_t * Address)
{
	int index;
	switch (AddressLength) {
		case 3 : {
			NRFWriteRegister(3,1); // 3 byte address length			
			break;
		}
		case 4 : {
			NRFWriteRegister(3,2); // 4 byte address length
			break;
		}
		case 5 : {
			NRFWriteRegister(3,3); // 5 byte address length
			break;
		}
		default: {
			return -1; // invalid address length
		}
	}
	NRFWriteCSN(0);	
	delay(10);
	NRFStatus = transferSPI(0x20+0x0b); // start write to RX_P0_Pipe address
	for (index = 0; index < AddressLength; index++)
	{
		NRFStatus = transferSPI(Address[index]);
		delay(10);
	}
	NRFWriteCSN(1);	
}

int NRFGetRXAddress(int MaxAddressLength, uint8_t * Address)
{
	int index;
	int actual_length;
	actual_length = NRFReadRegister(3);
	actual_length = actual_length + 2;
	if (actual_length > MaxAddressLength)
		return -1;
	NRFWriteCSN(0);
	delay(10);	
	NRFStatus = transferSPI(0x0a); // start read from RX_P0_Pipe address
	for (index = 0; index < actual_length; index++)
	{
		Address[index] = transferSPI(0xff);
		delay(10);
	}
	NRFWriteCSN(1);	
	return(0);
}
int NRFSetTXAddress(int AddressLength, uint8_t * Address)
{
	int index;
	switch (AddressLength) {
		case 3 : {
			NRFWriteRegister(3,1); // 3 byte address length			
			break;
		}
		case 4 : {
			NRFWriteRegister(3,2); // 4 byte address length
			break;
		}
		case 5 : {
			NRFWriteRegister(3,3); // 5 byte address length
			break;
		}
		default: {
			return -1; // invalid address length
		}
	}
	NRFWriteCSN(0);
	delay(10);	
	NRFStatus = transferSPI(0x20+0x10); // start write to TX address
	for (index = 0; index < AddressLength; index++)
	{
		transferSPI(Address[index]);
		delay(10);
	}
	NRFWriteCSN(1);	
	return(0);
}

int NRFGetTXAddress(int MaxAddressLength, uint8_t * Address)
{
	int index;
	int actual_length;
	actual_length = NRFReadRegister(3);
	actual_length = actual_length + 2;
	if (actual_length > MaxAddressLength)
		return -1;
	NRFWriteCSN(0);
	delay(10);	
	NRFStatus = transferSPI(0x10); // start read from TX address
	for (index = 0; index < actual_length; index++)
	{
		Address[index] = transferSPI(0xff);
		delay(10);
	}
	NRFWriteCSN(1);	
	return(0);
}
void NRFEnableTXMode()
{
	NRFWriteRegister(0,0x0a+BIT4+BIT5); // enable CRC, power up,  only RX causes IRQ
}
void NRFEnableRXMode()
{
	NRFWriteRegister(0,0x0b+BIT4+BIT5); // enable CRC, power up, RX mode, only RX causes IRQ
}
int NRFWriteData(int Length, uint8_t * Data)
{
	int index;
	if (Length > 32)
		return -1; // too long
	NRFWriteCE(0);
	NRFWriteRegister(0x07,0x70); // clear RX_DR,TX_DS,MAX_RT bits
	//delay(10);
	NRFEnableTXMode();
	//delay(10);
	NRFWriteCSN(0);	
	//delay(10);	
	NRFStatus = transferSPI(0xa0); // start write to TX buffer	
	for (index = 0; index < Length; index++)
	{	
		transferSPI(Data[index]);
		//delay(10);
	}	
	NRFWriteCSN(1);	
	//delay(10);
	NRFWriteCE(1);		
	//delay(10000);	// wait 10ms for packet to go
	//NRFEnableRXMode();
}
int NRFReadData(int MaxLength, uint8_t * Data)
{ // data is assumeed to be in data pipe 1
	int available_bytes;
	int index;
	int Length;
	available_bytes=NRFReadRegister(0x12); // find out how many bytes are available in P1
	if (available_bytes == 0)
		return 0;
	NRFWriteCSN(0);	
	NRFStatus = transferSPI(0x61); // start read from RX buffer
	if (available_bytes > MaxLength)
		Length = MaxLength;
	else
		Length = available_bytes;
	for (index = 0; index < Length; index++)
	{
		Data[index]=transferSPI(0xff);
	}
	NRFWriteCSN(1);	
	return Length;
}
